@@ -20,11 +20,11 @@ use POE;
 use aliased 'POE::Kernel' => 'K';
 
 # Public variables of the module.
-our $VERSION = '0.5.3';
+our $VERSION = '0.6.0';
 
 use base qw{ Class::Accessor::Fast };
 __PACKAGE__->mk_accessors( qw{
-    armies curplayer dst map move_in move_out nbdice src wait_for
+    armies curplayer dst got_card map move_in move_out nbdice src wait_for
     _players _players_active _players_turn_done _players_turn_todo
 } );
 
@@ -64,6 +64,23 @@ sub new {
 
 # -- public methods
 
+#
+# $game->cards_reset;
+#
+# put back all cards given to players to the deck.
+#
+sub cards_reset {
+    my ($self) = @_;
+    my $map = $self->map;
+
+    # return all distributed cards to the deck.
+    foreach my $player ( $self->players ) {
+        my @cards = $player->cards;
+        $map->card_return($_) for @cards;
+    }
+}
+
+
 #
 # $game->player_lost( $player );
 #
@@ -231,6 +248,10 @@ the current C<Games::Risk::Map> object of the game.
 
 =over 4
 
+=item * $game->cards_reset;
+
+Put back all cards given to players to the deck.
+
 
 =item * $game->player_lost( $player )
 
